@@ -1,7 +1,7 @@
 package Mail::Milter::Authentication::Handler;
 use strict;
 use warnings;
-our $VERSION = '2.20190523'; # VERSION
+our $VERSION = '2.20191106'; # VERSION
 
 
 use Digest::MD5 qw{ md5_hex };
@@ -1229,6 +1229,13 @@ sub destroy_object {
     # Objects may be set to not be destroyed,
     # eg. resolver and spf_server are not
     # destroyed for performance reasons
+    # Resolver, however, has its error cache cleared, as this should only
+    # cache errors within a single transaction.
+    if ($name eq 'resolver' ) {
+        if ( $thischild->{'object'}->{'resolver'}->can( 'clear_error_cache' ) ) {
+            $thischild->{'object'}->{'resolver'}->clear_error_cache();
+        }
+    }
     return if ! $thischild->{'object'}->{$name}->{'destroy'};
     return if ! $thischild->{'object'}->{$name};
     $self->dbgout( 'Object destroyed', $name, LOG_DEBUG );
@@ -2064,7 +2071,7 @@ Mail::Milter::Authentication::Handler
 
 =head1 VERSION
 
-version 2.20190523
+version 2.20191106
 
 =head1 DESCRIPTION
 
